
package wsclient;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ResourceBundle;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javax.websocket.ClientEndpointConfig;
import javax.websocket.CloseReason;
import javax.websocket.DeploymentException;
import javax.websocket.Endpoint;
import javax.websocket.EndpointConfig;
import javax.websocket.MessageHandler;
import javax.websocket.Session;
import org.glassfish.tyrus.client.ClientManager;

/**
 *
 * @author Cheney
 */
public class Controller implements Initializable {

    @FXML TextField urlField;
    @FXML TextArea rMsgText, sMsgText;
    @FXML Button connBut, disconnBut, sndBut;
    
    @Override
    public void initialize(URL location, ResourceBundle resources) {
        
    }
    
    public void connect(ActionEvent event) {

        try {
            session = client.connectToServer(endpoint, config, new URI(urlField.getText()));
        } catch (DeploymentException | IOException | URISyntaxException ex) {
            rMsgText.setText(ex.getMessage());
        }
    }
    
    public void disconnect(ActionEvent event) {

        try {
            session.close();
        } catch (IOException ex) {
            rMsgText.setText(ex.getMessage());
        }
    }
    
    public void send(ActionEvent event) {
        
        rMsgText.setText("");
        
        if(session != null && session.isOpen()) {
            try {
                session.getBasicRemote().sendText(sMsgText.getText());
            } catch (IOException ex) {
                rMsgText.setText(ex.getMessage());
            }
        }
    }
    
    private final Endpoint endpoint = new Endpoint() {

        @Override
        public void onOpen(Session session, EndpointConfig config) {
            session.addMessageHandler(msgHandler);
            Platform.runLater(() -> {
                urlField.setEditable(false);
                connBut.setDisable(true);
                disconnBut.setDisable(false);
                sndBut.setDisable(false);
            });
        }

        @Override
        public void onClose(Session session, CloseReason closeReason) {
            Platform.runLater(() -> {
                connBut.setDisable(false);
                disconnBut.setDisable(true);
                sndBut.setDisable(true);
                urlField.setEditable(true);
            });
        }
    };
    private final ClientEndpointConfig config = ClientEndpointConfig.Builder.create().build();
    private final ClientManager client = ClientManager.createClient();
    private Session session;
    private final MessageHandler.Whole<String> msgHandler = (msg) -> {
        Platform.runLater(() -> {
            rMsgText.setText(msg);
        });
    };
}
